#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>

#include "camera/Camera.h"

#include "car/camDataMsg.h"

PLUGINLIB_EXPORT_CLASS(car::Camera, nodelet::Nodelet);

namespace car
{

Camera::Camera()
    : messageOStream(nh, "Camera")
{}

void Camera::onInit()
{
    messageOStream.write("onInit", "START");

    camData = nh.advertise<camDataMsg>("camData", 1);
    cameraThread = std::thread(
        [this]()
        {
            if(!cap.isOpened()) {
                messageOStream << MessageOStream::key << "ERROR"
                               << MessageOStream::value << "No camera found!"
                               << MessageOStream::flush;
                while (true){ } // nothing will be send .... forever
            }
            cv::Mat tmp;
            cap >> tmp;
            int x_mid = tmp.cols/2;
            int y_mid = tmp.rows/2;

            for(;;) {
                cv::Mat orig;
                cap >> orig; // get a new frame from camera
                
                cv::Mat gray;
                cv::cvtColor(orig, gray, cv::COLOR_BGR2GRAY);
                cv::GaussianBlur(gray, gray, cv::Size(7,7), 1.5, 1.5);
                
                cv::Mat edges;
                cv::Canny(gray, edges, 0, 30, 3);
                
                std::vector<cv::Vec3f> circles;
                cv::HoughCircles(edges, circles, cv::HOUGH_GRADIENT, 1, 1, 100, 120);
                
                if (circles.size() > 0) {
                    cv::Point mean;
                    for( size_t i = 0; i < circles.size(); i++ ) {
                        cv::Point center(cvRound(circles[i][0]),
                                         cvRound(circles[i][1]));
                        mean += center;
                    }
                    
                    mean *= 1./circles.size();
                    float x_offset = (mean.x - x_mid)/((float)x_mid);
                    
                    /* messageOStream << MessageOStream::key << "x_offset" */
                    /*                << MessageOStream::value << x_offset */
                    /*                << MessageOStream::flush; */
                        
                    camDataMsg outMsg;
                    outMsg.xoffset = x_offset;
                    camData.publish(outMsg);
                }
            }
        });

    messageOStream.write("onInit", "END");
}
}
